﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using Telerik.WinControls.UI;
using NorthwindLibrary;
using Telerik.WinControls.Data;
using System.Reflection;
using Telerik.OpenAccess;
using System.ComponentModel;

namespace WinFormsApplication
{
    public class FormController
    {
        private CategoryForm view;
        private NorthwindOAEntityDiagrams context;

        public FormController()
        {
            this.SubscribeToEvents();

            this.context = ContextFactory.ObtainContext();
            this.View.CategorySource = this.context.Categories.ToList();
            this.View.ConfigureGrid();
        }

        public CategoryForm View
        {
            get
            {
                if (this.view == null)
                {
                    this.view = new CategoryForm();
                }
                return this.view;
            }
        }

        private void SubscribeToEvents()
        {
            this.View.DataSourceCollectionChanged += new ListChangedEventHandler(View_DataSourceCollectionChanged);
            this.View.RowsChanging += new GridViewCollectionChangingEventHandler(View_RowsChanging);

            this.View.SaveChangesButtonClicked += new EventHandler(View_SaveChangesButtonClicked);
            this.View.CategoryFormClosed += new FormClosedEventHandler(View_CategoryFormClosed);
        }

        private void View_DataSourceCollectionChanged(object sender, System.ComponentModel.ListChangedEventArgs e)
        {
            if (e.ListChangedType == ListChangedType.ItemAdded)
            {
                BindingSource source = sender as BindingSource;
                if (e.NewIndex >= 0 &&
                    e.NewIndex < source.List.Count)
                {
                    Category newCategory = source.List[e.NewIndex] as Category;
                    this.context.Add(newCategory);
                }
            }
        }

        private void View_RowsChanging(object sender, GridViewCollectionChangingEventArgs e)
        {
            if (e.Action == NotifyCollectionChangedAction.Remove)
            {
                DialogResult dialogResult = MessageBox.Show(
                    "Do you want to delete this category?", 
                    "Warning", 
                    MessageBoxButtons.OKCancel, 
                    MessageBoxIcon.Question);
                if (dialogResult != DialogResult.OK)
                {
                    e.Cancel = true;
                }
                else
                {
                    Category category = this.View.SelectedCategory;
                    if (category != null)
                    {
                        context.Delete(category);
                    }
                }
            }
        }

        private void View_SaveChangesButtonClicked(object sender, EventArgs e)
        {
            try
            {
                this.View.EndEdit();
                this.context.SaveChanges();
                DialogResult dialogResult = MessageBox.Show(
                    "Changes were successfully saved.",
                    "Result",
                    MessageBoxButtons.OKCancel,
                    MessageBoxIcon.Information);
            }
            catch (Exception ex)
            {
                string message = ex.Message;
                DialogResult dialogResult = MessageBox.Show(
                    string.Format("An error occurred during the save operation:\n{0}", message),
                    "Result",
                    MessageBoxButtons.OKCancel,
                    MessageBoxIcon.Error);
            }
        }

        private void View_CategoryFormClosed(object sender, FormClosedEventArgs e)
        {
            if (this.context != null)
            {
                this.context.Dispose();
            }
        }
    }
}
